"""
 context browser monitor
 part of conTEXT Amarok plugin
 (c) 2006 <camico@users.sourceforge.net>
 heavily based on the Amarok Smart DJ plugin
 (c) 2005 Rudd-O <dragonfear@gmail.com>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import os
import time	
import urllib
import re
import logging
import string

from ControllableThread import ControllableThread
from threading import Thread
from amarok import DcopAmarokFailed
from Observable import Observer

from common import LOGFILE,popup,tounicode,run
from FileProcessor import FileProcessor

class ContextBrowserUpdater(ControllableThread,Observer):

	logger = logging.getLogger("ContextBrowserUpdater")

	def __init__(self,proxyObject,configObject):
		ControllableThread.__init__(self)
		self.config = configObject
		self.amarok = proxyObject
		self.observe(self.amarok)
		self.observe(self.amarok.player)
		self.observe(self.amarok.contextBrowser)
		self.observe(self.config)
		self.amarok_available_yet = False
		self.update = False
		#self.current_box = "<div id='current_box' class='box'>"
		self.current_box = "<td id='current_box-largecover-td'>"
		self.jscode = file(os.path.join(os.path.dirname(__file__), "ContextBrowserUpdater.js")).read()
		self.jscode2 = file(os.path.join(os.path.dirname(__file__), "conTEXTmenu.js")).read()
		self.jscode2 = self.jscode2%( os.path.join(os.path.dirname(__file__), "img") )
		self.has_injected = 0
		self.fp = FileProcessor(self.amarok, self.config)
		self.manualsearchsite = ""
		self.searchnow = ""
		if self.config["autoSearchSites"] == "":
			self.config["autoSearchSites"] = self.fp.getInitialSites()
			
		# remove playlouder (conTEXT 2m)
		self.config["autoSearchSites"] = re.sub('playlouder.com:(False|True) ', '', self.config["autoSearchSites"])
		# remove bbc,dis,nme,stylus (conTEXT 2o)
		self.config["autoSearchSites"] = re.sub('bbc.co.uk:(False|True) ', '', self.config["autoSearchSites"])
		self.config["autoSearchSites"] = re.sub('drownedinsound.com:(False|True) ', '', self.config["autoSearchSites"])
		self.config["autoSearchSites"] = re.sub('nme.com:(False|True) ', '', self.config["autoSearchSites"])
		self.config["autoSearchSites"] = re.sub(' stylusmagazine.com:(False|True)', '', self.config["autoSearchSites"])
		
		self.browser = "kfmclient exec"
		try:
			amarokrc = file(re.sub('\.kde/share.*','.kde/share/config/amarokrc', __file__)).read()
			s = re.search("External Browser=(.*)", amarokrc)
			if s: self.browser = s.group(1)
		except:
			pass
		
	def processEvent(self,object,eventName,*args):
		if eventName == "amaroKExit":
			self.kill(asynchronous=True) # we kill ourselves
		if eventName == "amaroKAvailable":
			self.amarok_available_yet = True
		if eventName == "contentsChanged" and object == self.amarok.contextBrowser:
			self.update = True
		if eventName == "configChange":
			self.config_changed()
		if eventName == "stopped":
			self.has_injected = 0
	
	def run(self):
# 		while not self.__finish: time.sleep(1)
		try: self._run()
		except DcopAmarokFailed,e:
			self.logger.debug("Aborting because amaroK DCOP call failed: %s",e.output)
		except Exception,e:
			self.logger.exception("Something wrong in context browser monitor, process stopped %s %s",e,e.__class__)
			popup("Something went wrong in the context browser monitor loop.  This is not your fault, but a bug in the conTEXT plug-in.  Please report the contents of the file %s to the conTEXT plug-in developers.\n If this error persists, please delete the file %s and try again." %(os.path.abspath(LOGFILE),os.path.abspath("conTEXT.conf")))
		except:
			self.logger.exception("Something wrong in context browser monitor, process stopped")
			popup("Something went wrong in the context browser monitor loop.  This is not your fault, but a bug in the conTEXT plug-in.  Please report the contents of the file %s to the conTEXT plug-in developers.\n If this error persists, please delete the file %s and try again." %(os.path.abspath(LOGFILE),os.path.abspath("conTEXT.conf")))
	
	def _run(self):
		
		self.logger.debug("Context browser monitor starting")
		while not self.amarok_available_yet and not self.killed(): time.sleep(1)
		self.logger.debug("Context browser monitor started")
		
		try:
			version = self.amarok.player.call("version")
			if version.startswith("1.4"):
				self.amarok.contextBrowser.objectName = "html-widget1" # Music Tab
			elif version.startswith("1.3"):
				self.amarok.contextBrowser.objectName = "html-widget2" # Current Tab
				
		except DcopAmarokFailed:
			# old version!
			self.amarok.contextBrowser.objectName = "html-widget2"
		
		timer = 0
		self.has_injected = self.performInjection() # do once when we start
		
		while not self.killed():
			if self.update is False:
				
				if self.config["configPolling"]:
					timer += 1
					
					if self.has_injected > 0:
						self.pollCommands()
						
					if timer%5 is 0:
						if self.has_injected == 2:
							self.pollConfig()
				
				time.sleep(1)
				continue
					
			self.update = False # we do this so we can rerun it on the next iteration ASAP
			self.has_injected = self.performInjection()
			
		# exit
		self.fp.cleanup()
		self.amarok.contextBrowser.evalJS('undoInjection()')
		self.logger.debug("Context browser monitor exiting")

	def pollConfig(self):
		# poll config changes from javascript
		# this seems to be capable of crashing Amarok on exit :(
		
		if self.killed(): return
		jsconf = self.amarok.contextBrowser.call("evalJS", "getConfig()")
		
		if self.killed(): return
		try:
			boxheight, filename, nextsibling, shaded, expanded, scrollpos = jsconf.split(',')
		except ValueError:
			# the box wasnt there!
			return
		
		self.logger.debug(boxheight)
		self.config["maxBoxHeight"] = boxheight
		self.logger.debug(filename)
		self.config["fileName"] = filename
		self.logger.debug(nextsibling)
		self.config["insertBefore"] = nextsibling
		self.logger.debug(shaded)
		self.config["shaded"] = bool(shaded=="true")
		self.logger.debug(expanded)
		self.config["expanded"] = bool(expanded=="true")
		self.logger.debug(scrollpos)
		self.config["scrollPos"] = scrollpos
		
	def pollCommands(self):
		if self.killed(): return
		searchconf = self.amarok.contextBrowser.call("evalJS", "getCommands()")
		
		try:
			searchnow, file2trash, file2edit, url, self.manualsearchsite, autosearchsites = searchconf.split('|')
			self.logger.debug(self.manualsearchsite)
			
			if searchnow:
				self.fp.currentalbum = '<conTEXT:reinit>'
				self.searchnow = searchnow
				self.update = True
			if file2trash:
				self.logger.debug("trash file: %s", file2trash)
				run("kfmclient move", file2trash, "trash:/")
				self.update = True
			if file2edit:
				run("kfmclient exec", file2edit)
			if url:
				self.logger.debug("CLICKED URL: %s", url)
				run(self.browser, url)
			if self.manualsearchsite:
				self.fp.manualSearch(self.manualsearchsite)
				self.update = True
		
			#self.logger.debug(autosearchsites)
			if not self.searchnow == "SEARCH_ALL_TEMPORARY" or searchnow == "SEARCH_ALL_TEMPORARY":
				self.config["autoSearchSites"] = autosearchsites
		except ValueError:
			return
		
	
	def performInjection(self):
		
		def escape_js(string):
			return string.replace("\\","\\\\").replace("\"","\\\"")
		
		def convert_newlines(string):
			return string.replace("\r","").replace("\n","<br>")
		
		if self.killed(): return 0
		
		current_html = file(self.amarok.contextBrowser.getFilePath(),"rb").read(-1)
		if not self.current_box in current_html:
			self.logger.debug( "file is not current context browser, ignoring" )
			return 0
		
		if self.killed(): return 0
		self.logger.debug("file is current context browser, going forward with machiavellic plan")
		songfile = self.amarok.player.path()
		
		if not songfile: return 0
		self.logger.debug("Searching text file(s) for %s ",songfile)
		if self.killed(): return 0
		
		# get css code from contextbrowser.html
		s = re.search("(<style.*\/style>)", current_html, re.S)
		if s:
			css = s.group(1)
			s = re.search(".box\s*?{([^}]|[.])*?padding-bottom:\s*?(?P<pad>[0-9]+)px", css, re.S)
			if s: boxpadding = s.group('pad')
			else: 
				s = re.search(".box\s*?{([^}]|[.])*?padding:\s*?[0-9]+px\s*?[0-9]+px\s*?(?P<pad>[0-9]+)px", css, re.S)
				if s: boxpadding = s.group('pad')
				else:
					s = re.search(".box\s*?{([^}]|[.])*?padding:\s*?(?P<pad>[0-9]+)px", css, re.S)
					if s: boxpadding = s.group('pad')
					else: boxpadding = 0
		else:
			css = ""
			boxpadding = 0
		
		if self.killed(): return 0
		
		if self.config["configPolling"]:
			jsSearchMenuData = 'searchSites="%s"; hideDomains="%s"; largecover="%s";' %(self.config['autoSearchSites'], self.fp.getHideDomains(), self.amarok.player.call("coverImage").replace("cache","large").replace("130@",""))
		else:
			jsSearchMenuData = ""

		# look for files and reviews
		self.fp.setCSS(css)
		self.fp.reInit()
		self.config["autoSearchSites"] = self.fp.initSearchSites(self.config["autoSearchSites"])
		if not self.manualsearchsite:
			self.amarok.contextBrowser.evalJS(self.jscode2 + jsSearchMenuData)
			
		numlocal = self.fp.findFiles(songfile)
		numonline = 0
		
		artist = self.amarok.player.call("artist")
		album = self.amarok.player.call("album")
		if artist and album:
			numonline = self.fp.findOnlineReviews(artist, album)
		
			if self.manualsearchsite:
				if numonline == 0:
					return self.has_injected
				else:
					self.amarok.contextBrowser.evalJS('undoInjection()')
		else:
			if self.manualsearchsite:
				self.amarok.contextBrowser.evalJS('searchError("'+self.manualsearchsite+'", "missing tag")')
		
		if self.searchnow == "SEARCH_ALL_TEMPORARY":
			self.config["autoSearchSites"] = self.config["autoSearchSites"].replace("True", "False")
		self.searchnow = ""	

		if self.fp.numFiles() is 0:
			self.amarok.contextBrowser.evalJS('searchSites="'+self.config["autoSearchSites"]+'"')
			self.amarok.contextBrowser.evalJS('info(0)')
			return 1
		
		if self.fp.status is 1:
			self.config["expanded"] = True
		
		# statusbar message
		if not self.manualsearchsite:
			localfiles = ""
			if numlocal > 0:
				localfiles = " %s local file(s) and"%(numlocal)
			if not numonline == -1: # (same album - no search - no change - no message)
				self.amarok.playlist.shortStatusMessage("conTEXT found%s %s web review(s) for <b>%s</b>"%(localfiles, numonline, album))
		
		if self.killed(): return 0
		
		# encode/concatenate files
		originalfilenames = unicode("")
		filenames = unicode("")
		filetexts = unicode("")
		for i in range(self.fp.numFiles()):
			path = self.fp.fileName(i)
			opath = self.fp.originalFileName(i)
			text = self.fp.fileText(i)
			if i < self.fp.numFiles()-1: mark = '<conTEXT:mark>'
			else: mark = ''
			originalfilenames = originalfilenames + tounicode(opath) + mark
			filenames = filenames + tounicode(path) + mark
			filetexts = filetexts + tounicode(text).strip('\n\x0b\x0c\r').replace('\t','  ') + mark
		
		#filetexts = filetexts.encode('latin1')
		#filenames = filenames.encode('latin1')
		filetexts = escape_js(convert_newlines(filetexts))
		filenames = escape_js(filenames)
		originalfilenames = escape_js(originalfilenames)
		
		if( len(filetexts) + len(filenames) + len(originalfilenames) + len(self.jscode) + len(self.jscode2) + 100 > 128000 ):
			self.amarok.playlist.popupMessage("conTEXT:<br>Text file(s) too long! In maximized mode, something will be cut off.")
			cutpos = 128000 - len(filenames) - len(originalfilenames) - len(self.jscode) - len(self.jscode2) - 100
			filetexts = filetexts[:cutpos]
					
		self.logger.debug(self.config["autoSearchSites"])
		
		try:
			jscode = self.jscode%( boxpadding, filetexts, filenames, originalfilenames,
								self.config["maxBoxHeight"],
								self.config["expanded"],
								self.config["insertBefore"],
								self.config["fileName"],
								self.config["scrollPos"],
								self.config["shaded"],
								self.config["autoSearchSites"]
								)
		except:
			jscode = self.jscode%( boxpadding, filetexts, filenames, originalfilenames,
								self.config["maxBoxHeight"],
								self.config["expanded"],
								self.config["insertBefore"],
								"",
								self.config["scrollPos"],
								self.config["shaded"],
								self.config["autoSearchSites"]
								)
			
		if self.killed(): return 0
		self.amarok.contextBrowser.evalJS(self.jscode2+jscode)
		
		if self.manualsearchsite:
			self.amarok.contextBrowser.evalJS('searchSuccess("'+self.manualsearchsite+'")')
					
		return 2
				
	def config_changed(self):
		h = self.config["maxBoxHeight"]
		n = self.config["expanded"]
		self.amarok.contextBrowser.evalJS('refreshBox('+str(h)+',"'+str(n)+'")')
		
