"""
 class FileProcessor finds and processes text files and web reviews
 part of conTEXT Amarok plugin
 (c) 2006 <camico@users.sourceforge.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import os, re, logging, httplib, urllib, string
from common import popup
from Observable import Observer

ALBUM = 1
ARTIST = 2
	
class FileProcessor(Observer):

	timeout = 10
	logger = logging.getLogger("FileProcessor")
	extensionlist = [".txt",".Txt",".TXT",".htm",".Htm",".HTM",".html",".Html",".HTML",".php",".PHP",
					 ".nfo",".Nfo",".NFO","README","readme"]
	
	def __init__(self, proxyObject, configObject):
		self.amarok = proxyObject
		self.config = configObject
		self.observe(self.config)
		self.status = 0
		self.tmpdir = 'conTEXT-tmp'
		self.bodycode = ' onmouseover="if(!force)go=0" onmousemove="if(!force)go=0" id="body"'
		self.bodycode+= ' class="box" style="border:0px none;padding:0px;margin:0px;"'
		self.initialscrollcode = file(os.path.join(os.path.dirname(__file__), "Scrolling.html")).read()
		
		self.css = ''
		self.filenames = list()
		self.originalfilenames = list()
		self.currentalbum = ''
		self.manualsearchsite = ''
		
		self.sites = {}
		
		#
		# =========================
		# IF YOU WANT TO ADD A SITE - see the examples below
		# ==================================================
		# 
		# if available, use the site's own search, required array keys are:
		# - base: the base url
		# - search: the relative url for a search with %s for the artist name
		# - link: a regexp with a group matching the relative url for a hit with %s for the album name
		#
		# optional keys:
		# - artistlink: (if there are separate artist pages) a regexp with a group matching
		#               the relative url for a hit with %s for the artist name
		# - ...see examples
		#
		# or use google, required keys are:
		# - inurl
		
		#
		# then:
		# - create a class for the site - see bottom of this file
		# - and a unique string-matching case - see processFile()
		# - do some testing
		# - send me the patch!
		#
		 
		self.sites['allmusic.com'] = {}
		self.sites['allmusic.com']['base'] = "http://www.allmusic.com"
		self.sites['allmusic.com']['search'] = "/search/artist/%s" #"/cg/amg.dll?p=amg&sql=%s&opt1=1"
		self.sites['allmusic.com']['search_space_char'] = '-'
		#self.sites['allmusic.com']['artistlink'] = '(/cg/amg.dll\?p=amg&amp;sql=[^"]*)">%s( \[.*?\])?</a>'
		self.sites['allmusic.com']['artistlink'] = '(/artist/[^"]*)">%s( \[.*?\])?</a>'
		#self.sites['allmusic.com']['link'] = '(/cg/amg.dll\?p=amg&amp;sql=[^"]*)"><img src="[^"]*" alt="%s"'
		self.sites['allmusic.com']['link'] = '(/album/[^"]*)"><img width="[^"]*" border="0" title="%s'
		
		#self.sites['bbc.co.uk'] = {}
		#self.sites['bbc.co.uk']['base'] = "http://www.bbc.co.uk"
		#self.sites['bbc.co.uk']['search'] = "/cgi-bin/search/results.pl?q=%s&uri=/music/release/&go.x=0&go.y=0"
		#self.sites['bbc.co.uk']['search_with_album'] = True
		#self.sites['bbc.co.uk']['eliminate'] = ['<b>', '</b>', '<em class="searchterm">', '</em>']
		#self.sites['bbc.co.uk']['link'] = '%s by.*?<p class="displayUrl">www.bbc.co.uk(/music/reviews/[^<]*)</p>'
		
		self.sites['cduniverse.com'] = {}
		self.sites['cduniverse.com']['base'] = "http://www.cduniverse.com"
		self.sites['cduniverse.com']['search'] = "/sresult.asp?HT_Search_Info=%s&HT_Search=ARTIST&style=music"
		self.sites['cduniverse.com']['link'] = '(/productinfo.asp[^"]*)">%s'
		#self.sites['cduniverse.com']['extra_check'] = 'arrown.gif"> (Notes|Review|Customer Reviews)'
		self.sites['cduniverse.com']['extra_check'] = 'iheadmess">(CD Notes|Review)|<!---- trimable --->[^<]'
		
		#self.sites['drownedinsound.com'] = {}
		#self.sites['drownedinsound.com']['base'] = "http://www.drownedinsound.com"
		#self.sites['drownedinsound.com']['search'] = "/search?q=%s"
		#self.sites['drownedinsound.com']['search_with_album'] = True
		##self.sites['drownedinsound.com']['artistlink'] = '(/artist/view/[^"]*)">%s</a>'
		#self.sites['drownedinsound.com']['eliminate'] = ['<b>','</b>']
		#self.sites['drownedinsound.com']['link'] = '(/releases/[0-9]+/reviews/[0-9]+)">.*?%s / Releases'
		##self.sites['drownedinsound.com']['extra_check'] = "Our reviews|b_author";
		
		self.sites['guardian.co.uk'] = {}
		self.sites['guardian.co.uk']['inurl'] = "inurl:filmandmusic OR inurl:critic OR inurl:/music"
		self.sites['guardian.co.uk']['intitle'] = ALBUM
		self.sites['guardian.co.uk']['intitle2'] = "CD"
		#self.sites['guardian.co.uk'] = {}
		#self.sites['guardian.co.uk']['base'] = "http://www.guardian.co.uk"
		#self.sites['guardian.co.uk']['intitle'] = ALBUM
		
		#self.sites['laut.de'] = {}
		#self.sites['laut.de']['inurl'] = "inurl:lautstark"
		#self.sites['laut.de']['intitle'] = ALBUM
		self.sites['laut.de'] = {}
		self.sites['laut.de']['base'] = "http://www.laut.de"
		self.sites['laut.de']['search'] = "/Suche?suchbegriff=%s"
		#self.sites['laut.de']['post'] = "suchbegriff=%s"
		#self.sites['laut.de']['artistlink'] = 'a href="(/[^"]*)" class="wort"><b>%s</a>'
		self.sites['laut.de']['link'] = ' - \'%s\'" href="(/[^"]*)"'
		
		self.sites['metacritic.com'] = {}
		self.sites['metacritic.com']['base'] = "http://www.metacritic.com"
		self.sites['metacritic.com']['search_space_char'] = '+'
		#self.sites['metacritic.com']['search'] = "/search/process?sort=relevance&termtype=all&ts=%s&ty=2&x=0&y=0&button=search"
		#self.sites['metacritic.com']['search_with_album'] = True
		self.sites['metacritic.com']['search'] = "/search/all/%s/results"
		self.sites['metacritic.com']['artistlink'] = '(/person/[^"]*)">%s</a>'
		#self.sites['metacritic.com']['link'] = '(/music/artists/[^"]*)"><B>%s'
		self.sites['metacritic.com']['link'] = '(/music/[^"]*)">%s'
		
		self.sites['metal-observer.com'] = {}
		self.sites['metal-observer.com']['base'] = "http://www.metal-observer.com"
		self.sites['metal-observer.com']['search'] = "/search.php?q=%s&sa.x=0&sa.y=0"
		self.sites['metal-observer.com']['link'] = '<A href="http://www.metal-observer.com(/articles.php\?lid=1&sid=1&id=[0-9]*)"><STRONG>%s'

		#self.sites['nme.com'] = {}
		#self.sites['nme.com']['base'] = "http://www.nme.com"
		#self.sites['nme.com']['search'] = "/index.php?class=search&search_type=site&search_term=%s+reviews&search.x=0&search.y=0"
		#self.sites['nme.com']['eliminate'] = ['<b>','</b>','<strong>','</strong>']
		#self.sites['nme.com']['link'] = '%s - Album[^"]*"http://www.nme.com(/reviews/[^"]*)"'
		
		self.sites['pitchforkmedia.com'] = {}
		self.sites['pitchforkmedia.com']['base'] = "http://www.pitchfork.com"
		#self.sites['pitchforkmedia.com']['search'] = "/search/?query=%s&albums=1"
		self.sites['pitchforkmedia.com']['search'] = "/search/?query=%s&filters=albums&search_type=advanced"
		self.sites['pitchforkmedia.com']['link'] = '(/reviews/albums/[^"]*)"><strong>[^<]*</strong>: %s'

		self.sites['plattentests.de'] = {}
		self.sites['plattentests.de']['base'] = "http://www.plattentests.de"
		self.sites['plattentests.de']['search'] = "/suche.php?suche=%s&parameter=band"
		self.sites['plattentests.de']['eliminate'] = ['<b>', '</b>']
		self.sites['plattentests.de']['link'] = 'Du hast nach.*?(rezi.php\?show=[0-9]+)">[^<>]*? %s</a>'
		
		#self.sites['playlouder.com'] = {}
		#self.sites['playlouder.com']['base'] = "http://www.playlouder.com"
		#self.sites['playlouder.com']['search'] = "/find/artistsearch.html?searchtext=%s"
		#self.sites['playlouder.com']['link'] = '<a href="(/review[^"]*)">%s</a>'
		
		self.sites['progressiveworld.net'] = {}
		self.sites['progressiveworld.net']['inurl'] = "inurl:name=Reviews"
		self.sites['progressiveworld.net']['search_with_dash'] = True
		self.sites['progressiveworld.net']['extra_term'] = '"Back to reviews index"'
		#self.sites['progressiveworld.net']['intitle'] = "intitle:ProgressiveWorld.net"
		
		self.sites['punkbands.com'] = {}
		#self.sites['punkbands.com']['inurl'] = "inurl:reviews"
		##self.sites['punkbands.com']['intitle'] = "A Punk music review"
		self.sites['punkbands.com']['base'] = "punkbands.com"
		self.sites['punkbands.com']['search'] = "/bands/search"
		self.sites['punkbands.com']['artistlink'] = '(/bands/[^"]*)">%s</a>'
		self.sites['punkbands.com']['albumlink'] = '(/albums/[^"]*)">%s</a>'
		self.sites['punkbands.com']['link'] = '(/reviews/[^"]*)">read full review'
		self.sites['punkbands.com']['post'] = 'q=%s&a=search'
				
		self.sites['rollingstone.com'] = {}
		self.sites['rollingstone.com']['inurl'] = "inurl:reviews"
		self.sites['rollingstone.com']['intitle'] = ALBUM
		
		#self.sites['stylusmagazine.com'] = {}
		#self.sites['stylusmagazine.com']['base'] = "http://www.stylusmagazine.com"
		#self.sites['stylusmagazine.com']['search'] = "/search.php?search=%s&submit.x=0&submit.y=0"
		#self.sites['stylusmagazine.com']['link'] = "<a href='(/reviews/[^']*)'>%s</a>"
		#self.sites['stylusmagazine.com']['album_maxchars'] = 32
		
		try:
			os.mkdir(self.tmpdir)
		except OSError,e:
			if e.errno is 17: pass
			else:
				self.amarok.playlist.popupMessage("conTEXT could not create directory "+os.path.abspath(self.tmpdir)+
												  "<br>Only plaintext files in a full-size box will work.")
				self.status = 1
			
	def setCSS(self, css):
		self.css = css
		
	def reInit(self):
		self.cleanup()
		self.filenames = list()
		self.originalfilenames = list()
		self.filetexts = list()
		self.scrollpos = self.config["scrollPos"]
		self.currentfilename = self.config["fileName"]
				
	def findFiles(self, songfilename):
		self.reInit()
		self.songdir = os.path.dirname(songfilename)
		try:
			songdirfiles = os.listdir(self.songdir)
			songdirfiles.sort()
		except:
			return
			
		for file in songdirfiles:
			for ext in self.extensionlist:
				if file.endswith(ext):
					self.processFile(os.path.join(self.songdir,file))
					self.logger.debug("found file %s", file)
		
		if len(self.filenames) is 0:
			#self.amarok.playlist.shortStatusMessage("conTEXT did not find any text files.")
			self.logger.debug("no files")
		
		return len(self.filenames)
		
	def manualSearch(self, sitename):
		self.manualsearchsite = sitename
		
	def initSearchSites(self, autoSearchSites):
		s = autoSearchSites.split(" ")
		for siteconf in s:
			sitename, enabled = siteconf.split(":")
			if not self.sites.has_key(sitename): continue
			if enabled == "True":
				self.sites[sitename]["enabled"] = True
			else:
				self.sites[sitename]["enabled"] = False
				
		for sitename, site in self.sites.items():
			if not site.has_key("enabled"):
				site["enabled"] = False
				autoSearchSites += " " + sitename + ":False"
				
		autoSearchSites = autoSearchSites.strip()
		s = autoSearchSites.split(" ")
		s.sort()
		return string.join(s)
		
				
	def findOnlineReviews(self, artist, album):
		
		def get(url):
			self.logger.debug("%s", url)
			url = re.sub("http://","",url)
			host, url = url.split('/',1)
			
			h = httplib.HTTPConnection(host)
			h.putrequest('GET', "/"+url)
			h.putheader('Connection', 'close')
			h.putheader('Accept', '*/*')
			h.putheader('Referer', 'http://www.google.com/webhp?hl=en')
			
			h.endheaders()
			h.sock.settimeout(self.timeout)
			
			response = h.getresponse()
			self.logger.debug("%s, %s", response.status, response.reason)
			if response.status == 302 or response.status == 301:
				return get(response.getheader("Location"))
			
			data = response.read()
			#self.logger.debug("%s", data)
			return data
					
		def post(url,params):
			self.logger.debug("%s", url)
			url = re.sub("http://","",url)
			host, url = url.split('/',1)
			
			headers = { "Content-type": "application/x-www-form-urlencoded",
					"Accept": "text/html" }
			h = httplib.HTTPConnection(host)
			h.request('POST', "/"+url, params, headers)
			#h.putheader('Connection', 'close')
			#h.putheader('Accept', '*/*')
			#h.endheaders()
			h.sock.settimeout(self.timeout)
			
			response = h.getresponse()
			self.logger.debug("%s, %s", response.status, response.reason)			
			if response.status == 302 or response.status == 301:
				return post(response.getheader("Location"),params)
			
			data = response.read()
			#self.logger.debug("%s", data)
			return data
                              
		def mustHide(site):
			for domain in self.getHideDomains().split():
				if site.endswith(domain):
					return True
			return False;
			
		if self.currentalbum == album and self.config["saveOnlineReviews"] and not self.manualsearchsite:
			return -1
		
		self.currentalbum = album
		misses = list()
		hits = 0
		for sitename, site in self.sites.items():
			self.logger.debug(">>> %s", sitename + "+++" + self.manualsearchsite)
			if self.manualsearchsite:
				if sitename != self.manualsearchsite:
					continue
			else:
				if not site['enabled'] or mustHide(sitename):
					continue
			
			try:
				self.amarok.contextBrowser.evalJS('info(-1)')
				
				if site.has_key("inurl"):
					# ----------
					# use google
					
					if site.has_key("intitle"):
						if site['intitle'] == ALBUM:
							intitle = '+intitle:"'+urllib.quote(album)+'"'
						elif site['intitle'] == ARTIST:
							intitle = '+intitle:"'+urllib.quote(artist)+'"'
						else:
							intitle = '+"'+urllib.quote(site['intitle'])+'"'
						if site.has_key("intitle2"):
							intitle += '+intitle:"'+urllib.quote(site['intitle2']+'"')
					else:
						intitle = ""
						
					if site.has_key("search_with_dash"):
						searchstr = '"'+artist+' - '+album+'"'
					else:
						searchstr = '"'+artist+'"+"'+album+'"'
					
					if site.has_key("extra_term"):
						searchstr += '+' + site['extra_term']

					data = get("www.google.com/search?hl=en&source=hp&q=%s+site:%s+%s%s&btnI=%s&aq=f&oq=&aqi="%(
								urllib.quote(searchstr),
								urllib.quote(sitename),
								urllib.quote(site['inurl']),
								intitle,
								urllib.quote("I'm Feeling Lucky")
								))
					#data = post("www.google.com/search", {'q': "\"%s\"+\"%s\"+site:%s+%s%s"%(
								#urllib.quote(artist),
								#urllib.quote(album),
								#urllib.quote(sitename),
								#urllib.quote(site['inurl']),
								#intitle), 'btnI' : "I'm+feeling+lucky"}
								#)
								
					if 'did not match any documents' in data:
						misses.append(sitename)
						self.amarok.contextBrowser.evalJS('searchFailure("'+sitename+'")')
						continue
					if 'We apologize for the inconvenience' in data:
						misses.append(sitename + " (!)")
						self.amarok.contextBrowser.evalJS('searchFailure("'+sitename+'")')
						continue
								
				else:
					# ---------------------
					# use the site's search
					
					if site.has_key("search_with_album"):
						searchterm = urllib.quote(artist+" "+album)
					else:
						searchterm = urllib.quote(artist)
					
					if site.has_key("search_space_char"):
						searchterm = re.sub('%20', site['search_space_char'], searchterm)
						
					if site.has_key("post"):
						data = post( site['base']+site['search'], site['post']%(artist) )
					else:
						data = get( site['base']+site['search']%(searchterm) )
				
					if site.has_key("eliminate"):
						for item in site['eliminate']:
							data = re.sub(item, '', data)
					#self.logger.debug(data)
							
					if site.has_key("album_maxchars"):
						matchalbum = album[:site["album_maxchars"]] + '(?:...)?'
					else:
						matchalbum = album
						
					if site.has_key("artistlink"):
						#find artist link
						r = re.search(site['artistlink']%artist, data, re.S|re.I)
						if r:
							link = r.group(1)
							if link[:1] != '/': link = '/'+link
							link = re.sub('&amp;', '&', link)
							self.logger.debug(link)
							data = get(site['base']+link)
					
					if site.has_key("albumlink"):
						#find album link
						r = re.search(site['albumlink']%matchalbum, data, re.S|re.I)
						if r:
							link = r.group(1)
							if link[:1] != '/': link = '/'+link
							link = re.sub('&amp;', '&', link)
							self.logger.debug(link)
							data = get(site['base']+link)
							r = re.search(site['link'], data, re.S|re.I)
					else:
						#find review link:
						r = re.search(site['link']%matchalbum, data, re.S|re.I)
						
					if r:
						link = r.group(1)
						if link[:1] != '/': link = '/'+link
						link = re.sub('&amp;', '&', link)
						self.logger.debug(link)
						data = get(site['base']+link)
					else:
						misses.append(sitename)
						self.amarok.contextBrowser.evalJS('searchFailure("'+sitename+'")')
						continue
					
				if site.has_key("extra_check"):
					if not re.search(site['extra_check'], data, re.S):
						misses.append(sitename)
						self.amarok.contextBrowser.evalJS('searchFailure("'+sitename+'")')
						continue
				
			except Exception,e:
				self.logger.debug(e)
				self.amarok.playlist.shortStatusMessage("conTEXT could not connect to <b>%s</b>!"%sitename)
				self.amarok.contextBrowser.evalJS('searchError("'+sitename+'","no connection")')
				#self.manualsearchsite = ''
				#return -1
				continue
			
			
			hits += 1
			self.amarok.contextBrowser.evalJS('searchSuccess("'+sitename+'")')
			
			#title = "Review - %s" %sitename
			title = sitename
			if self.config["saveOnlineReviews"]:
				filename = os.path.join(self.songdir, title+".html")
			else:
				filename = os.path.join(os.path.abspath(self.tmpdir), title+".html.html")
			
			def writeFile():
				f = open(filename, 'w')
				f.write(data)
				f.close()
			
			try:
				writeFile()
			except:
				# no write permission - use tmp path
				filename = os.path.join(os.path.abspath(self.tmpdir), title+".html.html")
				writeFile()
				
			self.processFile(filename)
			
		if len(misses) > 0:
			self.amarok.playlist.shortStatusMessage("conTEXT could not find a review for <b>%s</b> on %s"%(album, string.join(misses, ', ')))
		
		self.manualsearchsite = ''
		return hits

	
	def processFile(self, filename):
		#filename = unicode(filename)
		file = open(filename, 'r')
		text = file.read()
		file.close()
		
		if False: pass
			
		# known review files
		elif '<TITLE>allmusic' in text or 'AllMusic</title>' in text:
			file = AllmusicFile(text)
			self.sites['allmusic.com']['enabled']=False
			
		elif 'AVERSIONLINE.COM</title>' in text:
			file = AversionlineFile(text)
			#self.sites['aversionline.com']['enabled']=False
			
		elif '<title>BBC' in text:
			file = BBCFile(text)
			#self.sites['bbc.co.uk']['enabled']=False
			
		elif 'content="Music, Movies and Games Shopping at CD Universe"' in text or 'https://www.cduniverse.com/checkout/' in text:
			file = CdUniverseFile(text)
			self.sites['cduniverse.com']['enabled']=False
                  
		elif '<title>Drowned in Sound - Reviews' in text or 'Drowned In Sound </title>' in text:
			file = DrownedInSoundFile(text)
			#self.sites['drownedinsound.com']['enabled']=False
			
		elif '<title>Guardian Unlimited Arts' in text or 'Guardian Unlimited Arts</title>' in text or '<div id="guardian-logo"' in text:
			file = GuardianFile(text)
			self.sites['guardian.co.uk']['enabled']=False
			
		elif '<title>laut.de' in text or 'window.laut' in text:
			file = LautFile(text)
			self.sites['laut.de']['enabled']=False
		
		elif 'Metacritic.com</title>' in text or 'About Metacritic</A>' in text:
			file = MetacriticFile(text)
			self.sites['metacritic.com']['enabled']=False
			
		elif 'at Metacritic</title>' in text:
			file = MetacriticFileV3(text)
			self.sites['metacritic.com']['enabled']=False
			
		elif '<TITLE>THE METAL OBSERVER' in text:
			file = MetalObserverFile(text)
			self.sites['metal-observer.com']['enabled']=False
			
		elif '<title>NME.COM - Reviews' in text or 'Album Reviews - NME.COM</title>' in text:
			file = NMEFile(text)
			#self.sites['nme.com']['enabled']=False
			
		elif 'Pitchfork Record Review</title>' in text or '<title>Pitchfork' in text:
			file = PitchforkmediaFile(text)
			self.sites['pitchforkmedia.com']['enabled']=False
			
		elif '<title>Plattentests' in text:
			file = PlattentestsFile(text)
			self.sites['plattentests.de']['enabled']=False
			
		elif '<title>PLAYLOUDER | review' in text or 'PLAYLOUDER</title>' in text:
			file = PlaylouderFile(text)
			#self.sites['playlouder.com']['enabled']=False
			
		elif '<TITLE>PROGRESSIVEWORLD.NET' in text or '<title>ProgressiveWorld.net' in text:
			file = ProgressiveWorldFile(text)
			self.sites['progressiveworld.net']['enabled']=False
			
		elif '<title>punkbands.com' in text or 'Punkbands dot com</title>' in text:
			file = PunkbandsFile(text)
			self.sites['punkbands.com']['enabled']=False
		
		elif '<title> Rolling Stone' in text or 'Rolling Stone</title>' in text or 'Rolling Stone Music | Music Reviews</title>' in text:
			file = RollingStoneFile(text)
			self.sites['rollingstone.com']['enabled']=False
		
		elif 'Stylus Magazine</TITLE>' in text:
			file = StylusMagazineFile(text)
			#self.sites['stylusmagazine.com']['enabled']=False
			
		elif 'sputnikmusic</title>' in text:
			file = SputnikFile(text)
			#self.sites['stylusmagazine.com']['enabled']=False
			
		# html file
		elif(filename.endswith(".htm") or
			 filename.endswith(".HTM") or
			 filename.endswith(".html") or
			 filename.endswith(".HTML") or
			 '<html>' in text or '<HTML>' in text):
			file = HTMLFile(text)
			
		# NFO file
		elif(filename.endswith(".nfo") or
			 filename.endswith(".NFO")):
			file = NFOFile(text)
			
		else:
			file = TextFile(text)
			file.wrap()
					
		if filename.endswith(".html.html"):
			filename = filename[:-5]
			
		self.writeTmpFile(file, filename)
	
	
	def writeTmpFile(self, file, filename):
		head = self.css + file.css
		self.logger.debug("currentfilename = %s", self.currentfilename )
		self.logger.debug("currentalbum = %s", self.currentalbum )
		if self.currentfilename == os.path.basename(filename) and self.currentalbum == self.amarok.player.call("album"):
			scrollpos = self.scrollpos
		else:
			scrollpos = 0
		scrollcode = self.initialscrollcode%( scrollpos, os.path.join(os.path.dirname(__file__), "img") )
				
		# replace links so that clicks become "pollable"
		if self.config["externalBrowser"]:
			file.text = re.sub('javascript: ', 'javascript:', file.text)
			links = re.findall('(?i)<a[^>]* href=["\']?([^"\'> ]*)["\']?[^>]*>', file.text)
			for link in links:
				full_link = link
				if not (link.startswith("http") or link.startswith("mailto")):
					if link.startswith("/"): full_link = link[1:]
					full_link = 'basehref()+'+'\'%s\''%full_link
				else:
					full_link = '\'%s\''%link
					
				link = re.sub('\?', '\\?', link)
				link = re.sub('\+', '\\+', link)
				link = re.sub('\|', '\\|', link)
				link = re.sub('\*', '\\*', link)
				if not re.match("javascript", link):
					link = re.sub('\(', '\\(', link)
					link = re.sub('\)', '\\)', link)
				#self.logger.debug(link)
				file.text = re.sub('(?i)<a[^>]* href=["\']?%s["\']?'%link, '<a href="javascript:parent.openURL(%s)"'%full_link, file.text)
				
		writetext = '<html><head>'+head+'</head><body'+self.bodycode+'>'+file.text+scrollcode+'</body></html>'
		
		# need abspath for the iframe later
		tmpfilename = os.path.join(os.path.abspath(self.tmpdir),os.path.basename(filename)) + '.html'
		try:
			tmpfile = open(tmpfilename, 'w')
			tmpfile.write(writetext)
			tmpfile.close()
			self.status = 0
		except IOError,e:
			if self.status is 0:
				self.amarok.playlist.popupMessage("conTEXT could not write to directory "+os.path.abspath(self.tmpdir)+
												  "<br>Only plaintext files in a full-size box will work.")
				self.status = 1
			
		if file.injectable:
			self.filetexts.append( file.unmodifiedtext )
			self.filenames.append( tmpfilename )
			self.originalfilenames.append( filename ) 
		else:
			if self.status is 0:
				self.filetexts.append( "" )
				self.filenames.append( tmpfilename )
				self.originalfilenames.append( filename ) 
		

	def fileText(self, index):
		return self.filetexts[index]
		
	def fileName(self, index):
		return self.filenames[index]
	
	def originalFileName(self, index):
		return self.originalfilenames[index]
	
	def numFiles(self):
		return len(self.filenames)
	
	def getInitialSites(self):
		s = ""
		for sitename, site in self.sites.items():
			s = s + sitename + ":False "
		return s.rstrip()
	
	def getHideDomains(self):
		hideDomains = ""
		if self.config["hideDotCom"]: hideDomains += '.com '
		if self.config["hideDotCoUk"]: hideDomains += '.uk '
		if self.config["hideDotDe"]: hideDomains += '.de '
		if self.config["hideDotNet"]: hideDomains += '.net '
		return hideDomains.rstrip()
		
	def cleanup(self):
		for file in self.filenames:
			try:
				os.remove(file)
				self.logger.debug("Removed temp file "+file)
			except OSError,e:
				self.logger.debug(str(e.errno)+": Could not remove temp file "+file)
				pass

class TextFile:
	
	injectable = False
	css = ""
	text = ""
	unmodifiedtext = ""
	
	def __init__(self, text):
		self.injectable = True
		self.unmodifiedtext = text
		
		text = re.sub("\r","",text)
		text = re.sub("<","&lt;",text)
		text = re.sub(">","&gt;",text)
		text = text.strip('\n\x0b\x0c\r').replace('\t','  ')
		self.text = '<div class="box-body">'+text+'</div>'
		
	def wrap(self):
		self.text = re.sub("\n","<br>",self.text)
	

class NFOFile (TextFile):
	def __init__(self, text):
		TextFile.__init__(self,text)
		self.text = re.sub('<div c', '<pre c', self.text)
		self.text = re.sub('</div>', '</pre>', self.text)
		
		self.css = """
			<style type="text/css">
				pre { font-family:Lucida ConsoleP; line-height:1em; }
			</style>
			"""

class HTMLFile (TextFile):
	def __init__(self, text):
		text = re.sub("<html.*>","",text);
		text = re.sub("<head.*>","",text);
		text = re.sub("<body.*>","",text);
		text = re.sub("</body>","",text);
		text = re.sub("</head>","",text);
		text = re.sub("</html>","",text);
		
		self.text = '<div id="wiki_box-body" class="box-body">'+text+'</div>'			
		
		self.css = """
			<style type="text/css">
				* {font-size:95%;}
			</style>
			"""
			
class PlattentestsFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		
		# new
		s = re.search("<div class=\"headerbox rezi\">(.*?)<\/div>", text, re.S)
		if s:
			tmptext += s.group(1)
			s = re.search("(<div id=\"rezitext\">.*?<\/div>)", text, re.S) # Rezitext
			if s: tmptext += s.group(1)
			s = re.search("(<div id=\"rezihighlights\">.*?<\/div>)", text, re.S) # Highlights
			if s: tmptext += s.group(1)
			s = re.search("(<div id=\"reziref\">.*?<\/div>)", text, re.S) # refs
			if s: tmptext += s.group(1)
			s = re.search("(<div id=\"surftipps\">.*?<\/div>)", text, re.S) # refs
			if s: tmptext += s.group(1)
			s = re.search("(<div id=\"reziweitere\">.*?)<div id=\"reziforum\">", text, re.S) # refs
			if s: tmptext += s.group(1)
			s = re.search("(<div id=\"reziforum\">.*?<\/div>)", text, re.S) # refs
			if s: tmptext += s.group(1)
		else:
			# old
			s = re.search("(<span class=\"reziheader\">.*<\/span><br>)", text) # Albumtitel
			if s: tmptext += s.group(1)
			s = re.search("(<span class=\"label\">.*<\/span><br>)", text) # Label
			if s: tmptext += s.group(1)
			s = re.search("(<span class=\"wertung\">.*<\/span>)", text) # VOE und Spielzeit
			if s: tmptext += s.group(1) + "<br>"
			s = re.search("(Unsere Bewertung: .*)<br>", text)
			if s: tmptext += s.group(1) + ", "
			s = re.search("(Eure.*-Bewertung:.*?<br>)", text, re.S)
			if s: tmptext += s.group(1) + "<br>"
			s = re.search("(<h1 align=\"center\" class=\"reziheader\">.*</h1>)", text) # Rezititel
			if s: tmptext += s.group(1)
			s = re.search("(<p align=\"left\" class=rezitxt>.*<\/p>)", text, re.S) # Rezitext
			if s: tmptext += s.group(1)
			s = re.search("(<span class=\"header\">Highlights:.*<br><br>)(<span|..<iframe)", text, re.S) # Highlights und Ref.
			if s: tmptext += s.group(1)
			s = re.search("(<span class=\"header\">Surftips:.*</span>)", text, re.S) # Surftips
			if s: tmptext += s.group(1)
		
		tmptext = re.sub("target=\"_blank\"","",tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<style type="text/css">
				body, p, a, h1, h2, h3, h4 {font-size:50%;}
				.reziheader {font-size:10px; font-weight:bold;}
				.bewertung {display:inline;}
				.header {font-weight:bold;}
				.label {font-style:italic;}
				.cover {display:none;}
			</style>
			<base href="http://www.plattentests.de/">
			"""
			
class PitchforkmediaFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		#s = re.findall("(<link href.*?>)", text) # css
		#for s in s: tmptext += s
		
		# title and data
		s = re.search("(<span class=\"reviewtitle.*?)<a", text, re.S)
		if s: tmptext += s.group(1)
              
		s = re.search("(<div class=\"article_body\">.*?<\/div>)", text, re.S) # review text
		if s: tmptext += s.group(1)
		s = re.search("(<img alt=\"Horizontal-dotbar-2col\".*?)<\/div>\s*<\/div>\s*<div id=\"article_column2\"", text, re.S) # related reviews and news
		if s: tmptext += s.group(1)
		else:
			# new
			s = re.search('(<div class="center column">.*?)<div class="right', text, re.S)
			if s: tmptext += s.group(1)
			tmptext = re.sub('<img src', '<img width="70" src', tmptext)
			
		tmptext = re.sub('(?s)(<a href="http://twitter.com/share".*?</iframe>)','',tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<style type="text/css">
				body, p, a, h1, h2, h3, h4 {font-size:50%;}
				img.record_pic {width:100px;}
				img.tombstone-cover-image {width:70px;}
				.share-link {display:none;}
			</style>
			<base href="http://www.pitchfork.com/">
			"""
			
class PlaylouderFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		#s = re.findall("(<link type.*?>)", text) # css
		#for s in s: tmptext += s
		s = re.search("(<div class=\"reviewheader\">.*?)<!--", text, re.S) # title and data
		if s: tmptext += s.group(1)
            
		tmptext = re.sub('<h.*?>', '', tmptext)
		tmptext = re.sub('</h.>', '', tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<style type="text/css">
			</style>
			<base href="http://www.playlouder.com/">
			"""
			
class NMEFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		text = re.sub('<div style="width: 614px;">', '<div>', text)
		text = re.sub('<img src="/layout/review_rating', '<img style="left:0;" src="/layout/review_rating', text)
		text = re.sub('<img src="/layout/user_rat', '<img style="position:absolute; left:-300px;" src="/layout/user_rat', text)
		text = re.sub('<h3>', '<br><h3>', text)
		
		s = re.search("(<h2>.*?add your own review</a>)", text, re.S) # title and data
		if s: tmptext += s.group(1)
		else:
			s = re.search('(<h1 class="headline".*?</h1>)', text) # headline
			if s: tmptext += s.group(1)
			s = re.search('(<span>[0-9]</span> out of 10)', text) # rating
			if s: tmptext += s.group(1)
			s = re.search('(<p class="sell">.*?)<div id="right_panel">', text, re.S) # text
			if s: tmptext += s.group(1)
		
		tmptext = re.sub('<li class="comments".*?</li>','',tmptext)
		tmptext = re.sub('(.+)<br />','\\1',tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<style type="text/css">
			   body, p, td, div, font, h1,h2,h3,h4 { font-size:50%; }
			</style>
			<base href="http://www.nme.com/">
                  <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
			"""
			
class DrownedInSoundFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		#s = re.findall("(<link href.*?>)", text) # css
		#for s in s: tmptext += s
		#s = re.findall("(<script src.*?>)", text) # js
		#for s in s: tmptext += s
		s = re.search("(<h3>.*?Post a user review</a></h3>)", text, re.S) # title and data
		if s: tmptext += s.group(1)
		else:
			s = re.search('(<h3>.*?)<div id="footad">', text, re.S) # title and data
			if s: tmptext += s.group(1)
			else:
				s = re.search('(<div class="header".*?)<div id="reply"', text, re.S) # title and data
				if s: tmptext += s.group(1)
				else:
					s = re.search('(<div class="content no-border".*?)<div class="panel">', text, re.S)
					if s: tmptext += s.group(1)
					s = re.search('(<div class="editorial".*?</ul>)', text, re.S) # title and data
					if s: tmptext += s.group(1)
					s = re.search('(<div class="comments_header".*?)<div class="sign_up', text, re.S)
					if s: tmptext += s.group(1)
					
			
		s = re.search('<a href="(/release/view/[0-9]*)\?type=user">', text)
		if s: tmptext = re.sub('<a href="#', '<a href="%s#'%s.group(1), tmptext)
		
		tmptext = re.sub('(?s)<form.*?/form>', '', tmptext)
		#tmptext = re.sub('(?i)<h..*?>|</h.>', '', tmptext)
		tmptext = re.sub('(?s)<div class="dis_btn">.*?</div>','',tmptext)
		tmptext = re.sub('<a.*?>\?</a>','',tmptext)
		tmptext = re.sub('<img.*?src="http://images.drownedinsound.com/resized_images.*?/>','',tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<meta http-equiv="Content-Type" content="text/html; charset=utf8">
			<base href="http://www.drownedinsound.com/">
			<style type="text/css">
				body, p, td, div, font, h3, h2, h1, h4 { font-size:50%; }
				.admin, .artist_tokens, .invisible, .add_comment { display:none; }
				#content img {display:none; }
			</style>
			"""
			
class StylusMagazineFile (TextFile):
	def __init__(self, text):
		self.css = """
			<base href="http://www.stylusmagazine.com/">
			<style type="text/css">
				body, p, td, div, font { font-size:50%; }
			</style>
			"""
		#s = re.search("(<LINK REL.*?>)", text) # css
		#if s: self.css += s.group(1)
		s = re.search("(<META HTTP.*?>)", text) # for utf8
		if s: self.css += s.group(1)
		
		tmptext = ""
		s = re.search("CONTENT -->(.*?)<!-- /CONTENT", text, re.S) # title and data
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('<div class=\'textbody\'>\s*<BR><BR>','<div>',tmptext)
		tmptext = re.sub('<br/>\s*<br/>', '<br/>', tmptext)
		tmptext = re.sub('(<P> )?(<img src=\'/images/big_.*?align=left)>','\\2 height=25>',tmptext)
		tmptext = re.sub('width=100 height=100','width=60 height=60 hspace=2 vspace=2',tmptext)
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
			
class GuardianFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		s = re.search('(<div id="(GuardianArticle|main-article-info)">.*?)(<br id="articleend">|<div id="related">)', text, re.S) # title and data
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('height="250"', 'height="0"', tmptext)
		tmptext = re.sub('Article continues', '', tmptext)
		tmptext = re.sub('(?s)</h1>.*?<BR>.*?<BR>', '</h1>', tmptext)
		tmptext = re.sub('(?i)<font.*?>|</font>', '', tmptext)
		tmptext = re.sub('<BR>\s*<BR>', '<br>', tmptext)
		tmptext = re.sub('(?i)<HR.*?>', '', tmptext)
		tmptext = re.sub('(?i)<B>(.*?The Guardian</a>)</B>', '\\1<br>', tmptext)
				
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.guardian.co.uk/">
			<style type="text/css">
                        body, p, td, div, font, h1,h2,h3,h4 { font-size:50%; }
				.image, .inline, .contributor-pic-small {display:none;}
				.rating-container { background-color: red; width:68px;}
				ul {list-style-type:none;}
				.tweet_button,.facebook,.reddit,.buzz {display:none}
			</style>
			"""

class PunkbandsFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		
		#old
		s = re.search('class="type3">(.*?)<form', text, re.S) # title and data
		if s: tmptext += s.group(1)
		
		s = re.search('(<h2.*?)<div class="side">', text, re.S)
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('href="javascript:;', '', tmptext)
		tmptext = re.sub('"images/ratings/(.).gif"', '"images/ratings/\\1.gif" width="40" alt="\\1 stars"', tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.punkbands.com/">
			<style type="text/css">
				.newspostedby, .newstext, .newscomments, .newstitle, h2, h3 { font-size:50%; }
				.user { display:block; font-weight:bold; }
			</style>
			"""

class BBCFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		
		s = re.findall('(<style type="text/css".*?/style>)', text) # css
		for s in s: tmptext += s
		
		# ver.1
		s = re.search('((?:<!-- )?<td valign="top" class="reviewtitle">.*?<tr><td>)<a name="comment">', text, re.S) # title and data
		if s: tmptext += s.group(1)
		else:
			# ver.3
			s = re.search('(<h3>Review</h3>.*?)<div id="nav">', text, re.S) # title and data
			if s:
				tmptext += s.group(1)
				s = re.search('(<h3>Key Tracks</h3>.*?)<!-- begin IPS',text,re.S)
				if s: tmptext += s.group(1)
			else:
				# ver.2
				s = re.search('<!-- End of GLOBAL NAVIGATION  -->(.*?)<div id="nav">', text, re.S) # title and data
				if s:
					tmptext += s.group(1)
				else:
					s = re.search('(<div id="summary".*?)<span class="mb_font"',text,re.S)
					if s: tmptext += s.group(1)
					s = re.search('(<div id="main_content">.*?)<a name="postcomment">',text,re.S)
					if s: tmptext += s.group(1)
					tmptext = re.sub('class="box">','>',tmptext)

					
		
		# add comment link at bottom
		s = re.search('name="uri" value="([^"]*)"', text) # page uri
		if s:
			uri = s.group(1)
			tmptext = re.sub('a href="#comment"', 'a href="'+uri+'#comment"', tmptext)
			tmptext += 'Have you heard this album? <a href="'+uri+'#comment">Tell us what you think</a>'
		
		# remove comment form (doesnt work)
		tmptext = re.sub('(?s)<a name="comment">.*?</form>.*?</div>', '<h4>Comments</h4>', tmptext)
		# remove misc stuff
		#tmptext = re.sub('<h1>.*?</h1>', '', tmptext)
		tmptext = re.sub('<(dt)/?>','&nbsp;&nbsp;',tmptext)
		tmptext = re.sub('<(dd)/?>','',tmptext)
		tmptext = re.sub('<img alt="Picture of:.*?>','',tmptext)
		
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.bbc.co.uk/">
			<style type="text/css">
				body, p, td, div, font, h1,h2,h3,h4,h5,h6 { font-size:50%; }
				.acs_name { font-weight:bold; }
			</style>
			"""

class RollingStoneFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		s = re.search('(<div class="header-top.*?)<div id="content-bottom">', text, re.S) # title and data
		if s: tmptext += s.group(1)
		else:
			s = re.search('(<div id="musicReview".*?)<!-- start: feature community review -->', text, re.S) # v2
			if s:
				tmptext += s.group(1)
				tmptext = re.sub('(?s)(<div id="mostPopular".*?)<div id="storyTextContainer"','<div id="storyTextContainer"',tmptext)
		
		# make ratings work (url hack)
		s = re.search('pageurl=(.*?)\;', text, re.S)
		if s:
			pageurl = s.group(1)
			s = re.search('<input type="hidden" name="rating_content_id".*?value="(.*?)"',tmptext,re.S)
			if s:
				content_id = s.group(1)
				tmptext = re.sub('function setRating\(id\) {', 'function setRating(id) {'+
																	'parent.openURL(basehref()+"'+pageurl+'?rating_content_id='+content_id+
																	'&rating_'+content_id+'_5="+id);',
																	tmptext)
			tmptext = re.sub('<form name="monorating"', '<form name="monorating" target="_blank"', tmptext)
																	
		tmptext = re.sub('height="250"', 'height="0"', tmptext)
		tmptext = re.sub('Average User Rating', '<br>Average User Rating', tmptext)
		tmptext = re.sub('of 5 Stars', ' of 5 Stars', tmptext)
		tmptext = re.sub('<P>', '', tmptext)
		tmptext = re.sub('<p class="artist">', '<span class="artist">', tmptext)
		tmptext = re.sub('<p class="albumtitle">', '<span class="albumtitle">', tmptext)
		tmptext = re.sub('</p></h1>', '</span></h1>', tmptext)
		tmptext = re.sub('Advertisement', '', tmptext)
		tmptext = re.sub('(?si)<IFRAME.*?</IFRAME>','',tmptext)
		tmptext = re.sub('(?si)<div class="tracklist">.*?<div class="simplefeature yourturnwide">','<div class="simplefeature yourturnwide">',tmptext)
		tmptext = re.sub('<img.*?</img>','',tmptext)
		tmptext = re.sub('(?s)<p class="useful".*?</p>','',tmptext)
		tmptext = re.sub('(?s)<p class="rate".*?</p>','<hr>',tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.rollingstone.com/">
			<style type="text/css">
				body, p, td, font, span, div, h1, h2, h3, h4 { font-size:50%; }
				.listen, .ad, .share-menu, .more-cdreviews, .user-rating { display:none }
				#newsandreviewswidget { display:none }
				.user-rating {margin:0;padding:0;border:2px solid; width:339px; height:34px;background-color:#906;}
				
				input[type="radio"] { width:12px; }
				ul { padding:0 }
				.headerImgHolder { display:none }
			</style>
			"""

class MetalObserverFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		s = re.search('(<TABLE border="0" cellpadding="0" cellspacing="0" width="100%"><TR>.*?)<script type="text/javascript">', text, re.S) # title and data
		if s: tmptext += s.group(1)
		s = re.search('(<TR height=150>.*?</TR>)', text, re.S) # tool box
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('COLOR: silver;|FONT-SIZE: 10pt;|FONT-FAMILY: Arial;', '', tmptext)
		tmptext = re.sub('<FONT.*?>', '', tmptext)
		
		tmptext += """<script>
			function printArticle(lid, sid, id) {
				parent.openURL('http://www.metal-observer.com/articles_print.php?lid=' + lid + '&sid=' + sid + '&id=' + id);
			}
			</script>"""
			
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.metal-observer.com/">
			<style type="text/css">
				body, p, td, div, font,span { font-size:50%; }
			</style>
			"""

class ProgressiveWorldFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		
		#v1
		s = re.search('<FONT FACE="Arial">(<STRONG>.*?</UL>\s?</STRONG></FONT></TD></TR>)', text, re.S) # title and data
		if s: tmptext += s.group(1)
		
		#v2
		s = re.search('(<font class="title">.*?)\[ <a href="modules\.php\?name=Reviews">Back to Reviews Index', text, re.S) # title and data
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('<BLOCKQUOTE/?>', '', tmptext)
		tmptext = re.sub('<center/?>', '', tmptext)
		tmptext = re.sub('<img src="images/reviews/.*?><br>', '', tmptext)
		
		# add score at top
		s = re.search('(<b>Score:</b>.*?<br>)', text, re.S)
		if s: tmptext = re.sub('<b>Year of Release', s.group(1) + '<b>Year of Release', tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.progressiveworld.net/html/">
			<style type="text/css">
				body, p, td, div, font { font-size:50%; }
			</style>
			"""

class LautFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		v2 = False
		#s = re.findall('(<style type="text/css".*?/style>)', text) # css
		#for s in s: tmptext += s
		s = re.search('<!-- cover plus info -->(.*?)<div (?:id|class)="spalteKommerz">', text, re.S) # title and data
		if s: tmptext += s.group(1)
		
		#v2
		s = re.search('<ul class="anzeigeBalken">(.*?)</ul>', text, re.S) # rating
		if s:
			tmptext += s.group(1) + '<br/>'
			v2 = True
		s = re.search('<h3 class="klappText">(.*?)<li class="widget" id="trackliste">', text, re.S)  # title and data
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('width="292"', 'width="230"', tmptext)
		tmptext = re.sub('margin-top: 37px;', 'margin-top:-20px; float:right;', tmptext)
		tmptext = re.sub('margin-top:12px;', 'margin-top:2px;', tmptext)
		tmptext = re.sub('id="cdInfo"|id="cdInfoButtons"|id="cdInfoText"|id="cdHead"', '', tmptext)
		tmptext = re.sub('leer.gif" height="12"', 'leer.gif" height="0"', tmptext)
		tmptext = re.sub('open\(url,\'leservoting.*', 'location.href=url;', tmptext)
		tmptext = re.sub('<img src="/images/lautstark/lautstark_button_preisvergleich.gif".*?>','', tmptext)
		tmptext = re.sub('height="(12|6)"', 'height="0"', tmptext)
		
		s = re.search('<img src="/images/analyse.gif\?USID\&ASID(/lautstark/cd-reviews/.*?)\&',text)
		if s:
			tmptext = re.sub('<a href="http://forum.laut.de/create_topic.php.*?>','<a href="%s">'%s.group(1), tmptext)

		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		if v2: self.css = '<meta http-equiv="Content-Type" content="text/html; charset=utf8">'
		else:  self.css = ''
		self.css += """
			<base href="http://www.laut.de/">
			<style type="text/css">
				.cdCover { display:none; }
				body, p, td, div, font, span, h1,h2,h3,h4 { font-size:50%; }
				
				/* v2 */
				#leserWertung { display:inline; }
				li { list-style-type:none; }
                        
#tracklist
{
  width: 140px;
  overflow: hidden;
  float: left;
  margin-right: 12px;
  margin-bottom: 4px;
}
#tracklist ul
{
  margin: 0px;
  list-style-type: none;
  padding: 4px 8px 8px 8px;
}

#tracklist ul li
{
  padding: 4px 0px 0px 0px;
}

#tracklist ul li img
{
  margin-left: 0px;
}

#tracklist ol
{
  list-style-position: outside;
  margin: 0px;
  padding: 4px 4px 4px 27px;
}

#tracklist ol li
{
  padding: 2px 0px 2px 0px;
  margin: 0px;
}

#tracklist ol li div
{
  margin-left: -2px;
}
			</style>
			"""

class AllmusicFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		#s = re.findall('(<link rel.*?>)', text) # css
		#for s in s: tmptext += s
		
		s = re.search('(<img src="[^"]*" alt="([0-9\.]* Stars).*?>)', text, re.S) # rating
		if s:
			tmptext += '<div>Rating</div>'
			tmptext += ' &nbsp;' + s.group(1) + ' &nbsp;(' + s.group(2) + ')<br>'
		else:
			#v2
			s = re.search('(<img src="/img/pages/site/stars/[^"]*".*?>)', text, re.S) # rating
			if s:
				tmptext += '<div>Rating</div>'
				tmptext += ' &nbsp;' + s.group(1) + '<br>'
		
		s = re.search('<!--Begin Center Content-->(.*?</p></td></tr></table></div>)', text, re.S) # text
		if s: tmptext += s.group(1) + '<br>'
		else:
			#v2
			s = re.search('<p class="text">(.*?)<div id="tracks">', text, re.S) # text
			if s:
				s2 = re.search('(<p class="author">.*?</p>)', text, re.S) # author
				if s2: tmptext += s2.group(1) + '<br style="clear:both">'
				tmptext += s.group(1) + '<br>'
		
		tmptext += '<div>Genre & Styles</div>'
		s = re.search('<!--Begin Genre.*?-->(.*?)<!--End Genre/Styles-->', text, re.S) # genres
		if s: tmptext += s.group(1)
		
		tmptext += '<div>Moods & Themes</div>'
		s = re.search('Moods( Listing|/Themes)-->(.*?)<!--End Moods/Themes-->', text, re.S) # moods
		if s: tmptext += s.group(2)
		
		s = re.search('</p></td></tr></table></div>(.*?)<!--End Center Content-->', text, re.S) # tracks
		if s: tmptext += s.group(1)
		else:
			#v2
			s = re.search('(<div id="tracks">.*?</div>)', text, re.S) # tracks
			if s:
				tmptext += s.group(1)
				tmptext = re.sub('width="235px"', '', tmptext)
				tmptext = re.sub('width="237px"', '', tmptext)
				tmptext = re.sub('width="552px"', '', tmptext)

		tmptext = re.sub('width="582"', '', tmptext)
		tmptext = re.sub('padding-bottom:5px', '', tmptext) # listings
		tmptext = re.sub('padding-bottom:4px', 'padding:3px', tmptext) # review text
		tmptext = re.sub('<img src="/i/pages/site/blocks.*?>', '', tmptext)
		tmptext = re.sub('<img src="/i/spacer.*?>', '', tmptext)
		tmptext = re.sub('</a></li>', '</a> </li>', tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.allmusic.com/">
			<style type="text/css">
				body, p, td, font, span, div, h2, th { font-size:50%; font-weight:normal; }
				li { display:inline; margin-left:1px; }
				ul { margin-left:-30px; }
				.title { padding-top:3px; }
				h3 { display:none; }
				.author { float:right; margin: 0; padding: 0}
			</style>
			"""

class AversionlineFile(TextFile):
	def __init__(self, text):
		tmptext = ""
		s = re.search('<!-- content START -->(.*?)<!-- content END -->', text, re.S) # text
		if s: tmptext += s.group(1)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.aversionline.com/">
			<style type="text/css">
				body, td { font-size:50%; }
			</style>
			"""

class MetacriticFileV3(TextFile):
	def __init__(self, text):
		tmptext = ""
		
		s = re.search('(<div class="summary_wrap".*?)<div id="side"', text, re.S) # text
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('(?s)(<div class="label">Your Score.*?</ul>)', '', tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<meta http-equiv="Content-Type" content="text/html; charset=utf8">
			<base href="http://www.metacritic.com/">
			<style type="text/css">
				body, td, h1, h2, h3 { font-size:50%; }
				.product_details { border-top: 1px dotted gray }
				.author_reviews { display:none }
				.review_helpful { display:none }
				.review_grade { float:right }
				.review_critic { float:left }
				.review_body { clear:both }
				.review_content { border-top: 1px dotted gray }
				li { list-style-type: none; }
				ul,ol { padding-left:10px; }
			</style>
			"""

class MetacriticFile(TextFile):
	def __init__(self, text):
		tmptext = ""
		#s = re.findall('(<LINK HREF.*?>)', text) # css
		#for s in s: tmptext += s
		
		#v1
		s = re.search('<H1>(.*?Discuss this album in our forums</A>)', text, re.S) # text
		if s: tmptext += s.group(1)
		
		#v2
		s = re.search('<h1>(.*?)<!-- end comments -->', text, re.S) # text
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('PRINT|EMAIL','',tmptext)
		tmptext = re.sub('User Score:','',tmptext)
		tmptext = re.sub('<a.*?Read user comments</a><br />','',tmptext)
		tmptext = re.sub('<a.*?Rate this album &gt;</a>','',tmptext)
		tmptext = re.sub('<a href="#critics">(.*?)</a>','\\1',tmptext)
		tmptext = re.sub('<a.*?Read more user comments &gt;</a>','',tmptext)
		tmptext = re.sub('All critic scores are.*?Learn more...</a>','',tmptext)
		tmptext = re.sub('How did we calculate this\?','',tmptext)
		
		tmptext = re.sub('(?s)<TD WIDTH="150"><IMG.*?</TD>','<td></td>',tmptext)
		tmptext = re.sub('(?s)<iframe.*?</iframe>','',tmptext)
		
		#tmptext = re.sub("(?s)A HREF=\"javascript:;\" onClick=\"window.open\('(.*?)'.*?\">",
						  #'a href="\\1;">', tmptext)

		s = re.search('<A HREF="/print(/music/.*?)">', text, re.S)
		if s: tmptext = re.sub('<A HREF="javascript:.*?>','<A HREF="%s">'%s.group(1), tmptext)
		
		tmptext = re.sub('<B><A HREF="#critics">Read critic reviews</A></B><BR>','',tmptext)
		tmptext = re.sub('<B><A HREF="#users">Read user  comments</A></B><BR>','',tmptext)
		tmptext = re.sub('<IMG SRC="/_images/heads/.*?" ALT="(.*?)".*?>','\\1',tmptext)		
		tmptext = re.sub('<IMG SRC="/_images/scores/outof.*?>', '',tmptext)
		tmptext = re.sub('<IMG SRC="/_images/scores.*?" ALT="(.*?)".*?>', '\\1',tmptext)
		tmptext = re.sub('<TD ALIGN="CENTER" CLASS="(green|yellow|red)">(.*?)</TD>', '<TD><b><span class="\\1">\\2</span></b> out of 10</TD>',tmptext)
		tmptext = re.sub('TD WIDTH="50"','TD',tmptext)
		tmptext = re.sub('CELLSPACING="10"','CELLSPACING="5"',tmptext)
		tmptext = re.sub('<BR>\s*<SPAN CLASS="subhead">',' ',tmptext)
		
		s = re.search('Metascore: ([0-9]*)',tmptext)
		if s:
			score = int(s.group(1))
			if score > 60: scoreclass = "green"
			elif score >= 40: scoreclass = "yellow"
			else: scoreclass = "red"
		else:
			scoreclass = ""
			
		tmptext = re.sub('Metascore: ([0-9]*)','<b><span class="%s">\\1</span></b> out of 100'%scoreclass, tmptext)
		
		tmptext = re.sub('<IMG SRC="/_images/readreview.gif" ALT="(.*?)".*?>','\\1',tmptext)
		tmptext = re.sub('<IMG SRC="/_images/button_votenow.*?>','',tmptext)
		tmptext = re.sub("TARGET=\"_blank\"","",tmptext);
				
		#v1
		s = re.search('(?s)(<DIV ID="rightlinks">.*?</DIV>)', tmptext)
		if s:
			productlinks = s.group(1)
			tmptext = re.sub('(?s)(<DIV ID="rightlinks">.*?</DIV>)', '', tmptext)
			tmptext += "<hr>"+productlinks
		
		#v2
		s = re.search('(?s)(<div id="productlinks">.*?</div>)', tmptext)
		if s:
			productlinks = s.group(1)
			tmptext = re.sub('(?s)(<div id="productlinks">.*?</div>)', '', tmptext)
			tmptext += "<hr>"+productlinks
		
		tmptext = re.sub('<script.*?</script>','',tmptext)
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<meta http-equiv="Content-Type" content="text/html; charset=utf8">
			<base href="http://www.metacritic.com/">
			<style type="text/css">
				body, td, h1, h2, h3 { font-size:50%; }
				div.greenscorerow {
					border-left: solid 30px #33CC00;
					border-bottom: solid 0px #33CC00;
					padding: 2px 1px;
					margin-top:10px;
				}
				div.yellowscorerow {
					border-left: solid 30px #FFFF00;
					border-bottom: solid 0px #FFFF00;
					padding: 2px 1px;
					margin-top:10px;
				}
				div.redscorerow {
					border-left: solid 30px #FF0000;
					border-bottom: solid 0px #FF0000;
					padding: 2px 1px;
					margin-top:10px;
				}
				div.criticscore {
					left: -15px;
					width: 35px;
					text-align: right;
					float: left;
					position: absolute;
					padding-top: 4px;
					font-weight:bold;
				}
				.green, .greencell {
					font-weight: bold;
					color: #000000;
					background-color: #33CC00;
					padding-right: 3px;
					padding-left: 3px;
					letter-spacing: 0px;
					display:inline;
				}
				.yellow, .yellowcell {
					font-weight: bold;
					color: #000000;
					background-color: #FFFF00;
					padding-right: 3px;
					padding-left: 3px;
					letter-spacing: 0px;
					display:inline;
				}
				.red, .redcell {
					font-weight: bold;
					color: #FFFFFF;
					background-color: #FF0000;
					padding-right: 3px;
					padding-left: 3px;
					letter-spacing: 0px;
					display:inline;
				}
				.noscore, .noscorecell {
					font-weight: bold;
					color: #000000;
					background-color: #808E9B;
					padding-right: 3px;
					padding-left: 3px;
					letter-spacing: 0px;
				}
				#bigpic, button {
					display:none;
				}
				#metascore, #userscore, .entity {
					display:inline;
				}
				#userscore:before {
					content: 'Userscore: '; 
				}
				#scoredescription {
					margin-left: -3px;
				}
				#scores:before {
					content: 'Metascore: '; 
				}
				#scoredescription, #scoreinfo {
					background: none; font-weight:normal; display:inline;
				}
				p {
					margin:0px;
				}
				.reviewbody, .publication, .quote {display:inline}
				.review { margin-bottom: 5px; }
				.publication:after { content: ':'; }
			</style>
			"""

class CdUniverseFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		s = re.search("<title>(.*?) CD</title>", text)
		if s: tmptext += s.group(1) + "<br>"
		
		s = re.search('arrown.gif"> (Notes.*?<div align="?justify"?>.*?</div>)', text, re.S) # notes
		if s: tmptext += s.group(1)
		s = re.search('iheadmess">(CD Notes.*?<div align="?justify"?>.*?</div>)', text, re.S) # notes v2
		if s: tmptext += s.group(1)
		
		s = re.search('arrown.gif"> (Review.*?<div align="?justify"?>.*?</div>)', text, re.S) # review
		if s: tmptext += s.group(1)
		#s = re.search('iheadmess">(Review.*?<div align="?justify"?>.*?</div>)', text, re.S) # review v2
		#if s: tmptext += s.group(1)
		s = re.search('<!---- trimable --->(.*?)<!---- trimable --->', text, re.S) # review etc 3
		if s: tmptext += s.group(1)
		
		s = re.search('arrown.gif"> (Customer Reviews.*?<div align="?justify"?>.*?</div>)', text, re.S) # customer reviews
		if s: tmptext += s.group(1)
		s = re.search('arrown.gif"> (Customer  <span class="iheadmess">.*?<div align="?justify"?>.*?</div>)', text, re.S) # customer reviews v2
		if s: tmptext += s.group(1)
		
		tmptext = re.sub('(CD )?Notes', '<b>Notes</b>', tmptext)
		tmptext = re.sub('Customer Reviews', '<br><b>Customer Reviews</b>', tmptext)
		tmptext = re.sub('<br>Was This Review Helpful.*?No</a>', '', tmptext)
		tmptext = re.sub('Customer (.*?) Reviews', '<br><b>Customer Reviews</b>', tmptext)
		tmptext = re.sub('Review</span>', 'Review ', tmptext)
		tmptext = re.sub('Review[^s]', '<br><b>Review</b>', tmptext)
		tmptext = re.sub(' align="?justify"?', '', tmptext)
		
		tmptext += "<script>document.getElementById('NoteSnippet').style.display='inline'; document.getElementById('NoteSnippet').style.visibility='visible';</script>"
		
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.cduniverse.com/">
			<style type="text/css">
				.notimportantlink  {display:none}
			</style>
			"""

class SputnikFile (TextFile):
	def __init__(self, text):
		tmptext = ""
		
		s = re.search("<table cellpadding=4 cellspacing=4 width=545>(.*?)<font size=2>Share", text, re.S)
		if s: tmptext += s.group(1)
				
		self.text = '<div id="wiki_box-body" class="box-body">'+tmptext+'</div>'
		
		self.css = """
			<base href="http://www.sputnikmusic.com/">
			<style type="text/css">
				body, td, h1, h2, h3, div, font { font-size:50%; }
			</style>
			"""
